#!/bin/bash
#

# buildLhapdf.sh 

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <c2|c7|e17|e19> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=lhapdf
origpkgver=v6_2_3
pkgver=${origpkgver}
ssibuildshims_version=v1_04_13
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=LHAPDF-${pkgdotver}

get_this_dir

get_ssibuildshims

source define_basics --
scriptdir=${product_dir}/${package}/${pkgver}/scripts

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    cxxflg="-O3 -g -DNDEBUG"
elif [ "${extraqual}" = "debug" ]
then
    cxxflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cxxflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

if [[ "${basequal}" == e[679] ]]
then
  cc=gcc
  cxx=g++
  cxxflg="${cxxflg} -std=c++1y -Wno-deprecated-declarations"
elif [[ "${basequal}" == e1[045] ]]
then
  cc=gcc
  cxx=g++
  cxxflg="${cxxflg} -std=c++14 -Wno-deprecated-declarations"
elif [[ "${basequal}" == e1[79] ]]; then
  cc=gcc
  cxx=g++
  cxxflg="${cxxflg} -std=c++17 -Wno-deprecated-declarations"
elif [[ "${basequal}" == c[27] ]]; then
  cc=clang
  cxx=clang++
  cxxflg="${cxxflg} -std=c++17 -Wno-deprecated-declarations"
else
  ssi_die "Qualifier $basequal not recognized."
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} 

if [ -z ${LHAPATH} ]
then
   echo "ERROR: failed to setup lhapdf"
   exit 1
else
   echo "LHAPATH: ${LHAPATH}"
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

if [ ! -d ${sourcedir}/${pkgtarname} ]
then
   cd ${sourcedir}
   tar xf ${tardir}/${pkgtarname}.tar.gz
   # get updated config files
   cp -p ${patchdir}/config.* ${sourcedir}/${pkgtarname}/config/
fi

cd ${blddir}

CC=${cc} CXX=${cxx} FC=gfortran \
CXXFLAGS="${cxxflg}" ${sourcedir}/${pkgtarname}/configure \
 --prefix=${pkgdir} \
 --disable-python || \
 ssi_die "ERROR: configure failed."
env CC=${cc} CXX=${cxx} FC=gfortran make -j ${ncore} || ssi_die "ERROR: make failed."
env CC=${cc} CXX=${cxx} FC=gfortran make check || ssi_die "ERROR: make check failed"
env CC=${cc} CXX=${cxx} FC=gfortran make install || ssi_die "ERROR: make install failed."

# install example source code by hand
cp -pr ${sourcedir}/${pkgtarname}/examples ${pkgdir}/examples
rm -f ${pkgdir}/examples/Makefile.*
rm -f ${pkgdir}/examples/*.py
cp -p ${blddir}/examples/* ${pkgdir}/examples/

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || exit 1

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "lhapdf is installed at ${LHAPDF_FQ_DIR}"

# fixing bin/lhapdf-config
mv ${pkgdir}/bin/lhapdf-config ${pkgdir}/bin/lhapdf-config.bak
cat ${pkgdir}/bin/lhapdf-config.bak | sed "s%${LHAPDF_FQ_DIR}%\${LHAPDF_FQ_DIR}%g" > ${pkgdir}/bin/lhapdf-config

# remove libtool files since they contain hardwired paths
rm -vf ${LHAPDF_FQ_DIR}/lib/*.la

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
